/*------------------------------------------------------------------------
 Module:        output_mp3.c
 Author:        (c) 2003 Felix de las Pozas Alvarez
 Project:       Soul Reaver 1 music extractor and converter
 State:         v1.1 FINAL VERSION
 Creation Date: December 2 2003
 Description:   Write the files in MP3 format. Source file.
 License:       GNU Public License 
 
    This file is part of SR_Music.
    SR_Music free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    SR_Music is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with SHA-1; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
    USA
------------------------------------------------------------------------*/

#include <io.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "lame/include/lame.h"

void mp3_output(FILE *musicdat)
{
    /* songs positions in music.dat */
    const unsigned long POSITION[] =
        {
            0,
            9838576,
            23828720,
            33274480,
            43545200,
            53122640,
            63134608,
            76862256,
            92508192,
            101179808,
            111809812
        };

    FILE *output, *playlist;
    unsigned char output_name[120];
    unsigned char name_temp[120];
    unsigned char pcm_buffer[4096];
    unsigned short int pcm_right[1024], pcm_left[1024];
    unsigned char mp3_buffer[8480];
    unsigned short int loop, loop2;
    unsigned long size;
    unsigned long bytes;

    lame_global_flags *gfp;

    /* try to open playlist file, notify errors and exit */
    if (NULL == (playlist = fopen("Kurt Harland - Soul Reaver.m3u", "wb")))
    {
        printf("ERROR: Error creating playlist file.\n");
        exit(-1);
    }

    for (loop = 0; loop < 10; loop++)
    {
        /* create name & write it to the playlist */
        strcpy(output_name, "Track ");
        itoa(loop+1, name_temp, 10);
        strcat(output_name, name_temp);
        strcpy(name_temp, output_name);
        strcat(output_name, ".mp3");

        fputs(output_name, playlist);
        fputs("\n", playlist);

        /* try to open output file, notify errors and exit */
        if (NULL == (output = fopen(output_name, "wb")))
        {
            printf("ERROR: Error opening output file %s\n",output_name);
            exit(-1);
        }

        size = POSITION[loop+1] - POSITION[loop];
        fseek(musicdat, POSITION[loop], SEEK_SET);

        printf("Writting file : \"%s\" ... ", output_name);

        /* mp3 init stuff */
        gfp = lame_init();
        lame_set_num_channels(gfp,2);
        lame_set_in_samplerate(gfp,22050);
        lame_set_brate(gfp,128);
        lame_set_mode(gfp,0);
        lame_set_quality(gfp,0);
        lame_set_bWriteVbrTag(gfp,0);

        /* add comments to the song */
        id3tag_v1_only(gfp);
        id3tag_set_title(gfp,name_temp);
        id3tag_set_artist(gfp,"Kurt Harland");
        id3tag_set_album(gfp, "Soul Reaver Soundtrack");
        id3tag_set_comment(gfp,"(C) Eidos Interactive");
        itoa(loop+1, output_name, 10);
        id3tag_set_track(gfp,output_name);
        id3tag_set_genre(gfp,"Soundtrack");

        if (0 > (bytes = lame_init_params(gfp)))
        {
            printf("\nERROR: Error in LAME init stage : %ld.\n", bytes);
            exit(-1);
        }

        while(size > 0)
        {
            /* read PCM samples in 4096 blocks if we can */
            if (size > 4096)
            {
                /* read PCM interleaved samples */
                if (4096 != (bytes = fread(pcm_buffer,1,4096,musicdat)))
                {
                    printf("\nERROR: I/O Error reading music.dat\n");
                    exit(-1);
                }
                size -= 4096;
            }
            else
            {
                if (size != (bytes = fread(pcm_buffer,1,size,musicdat)))
                {
                    printf("\nERROR: I/O Error reading music.dat\n");
                    exit(-1);
                }
                size = 0;
            }

            printf("%3d %%\b\b\b\b\b", (int) ((((POSITION[loop+1] - POSITION[loop]) - size) * 100) / (POSITION[loop+1] - POSITION[loop])));

            if (bytes != 0)
            {
                /* there are samples to encode */

                /* uninterleave samples for MP3 (short int format) */
                for(loop2 = 0; loop2 < bytes/4 ; loop2++)
                {
                    pcm_left[loop2]  = ((pcm_buffer[loop2*4+1]<<8) | pcm_buffer[loop2*4]);
                    pcm_right[loop2] = ((pcm_buffer[loop2*4+3]<<8) | pcm_buffer[loop2*4+2]);
                }

                /* mp3 encode */
                if (0 > (bytes = lame_encode_buffer(gfp, pcm_left, pcm_right, 1024, mp3_buffer, 8480)))
                {
                    printf("\nERROR: Error in the LAME encode process : %ld.\n", bytes);
                    exit(-1);
                }

                /* write encoded samples */
                if (bytes != fwrite(mp3_buffer, 1, bytes, output))
                {
                    printf("\nERROR: Error writting mp3 output.\n");
                    exit(-1);
                }
            }
        }

        /* flush mp3 buffer with last frames */
        if (0 > (bytes = lame_encode_flush(gfp, mp3_buffer, 8480)))
        {
            printf("\nERROR: Error in the mp3 buffer flush : %ld.\n", bytes);
            exit(-1);
        }

        /* write last samples */
        if (bytes != fwrite(mp3_buffer, 1, bytes, output))
        {
            printf("\nERROR: Error writting mp3 output.\n");
            exit(-1);
        }

        /* close the output file */
        if (fclose(output) == EOF)
        {
            printf("\nERROR: Error closing output file %s\n", output_name);
        }
        printf("OK!  \n");

        lame_mp3_tags_fid(gfp, NULL);

        /* clean up */
        lame_close(gfp);

    }

    printf("Writting M3U playlist ... OK!\n");

    /* close the playlist ile */
    if (fclose(playlist) == EOF)
    {
        printf("ERROR: Error closing playlist file.");
    }

    return;
}

