/*------------------------------------------------------------------------
 Module:        output_wav.c
 Author:        (c) 2003 Felix de las Pozas Alvarez
 Project:       Soul Reaver 1 music extractor and converter
 State:         v1.1 FINAL VERSION
 Creation Date: December 2 2003
 Description:   Write the files in PCM WAV format. Source file.
 License:       GNU Public License 
 
    This file is part of SR_Music.
    SR_Music free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    SR_Music is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with SHA-1; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
    USA
------------------------------------------------------------------------*/

#include <io.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

/* WAV file header for writting simple PCM WAV files */
typedef struct
{
    unsigned long  ChunkID;   /* RIFF */
    unsigned long  ChunkSize;  /* variable, will set this later */
    unsigned long  Format;   /* WAVE */
    unsigned long  SubChunk1ID;  /* fmt  */
    unsigned long  SubChunk1Size; /* 16 for PCM */
    unsigned short AudioFormat;  /* 1 for PCM */
    unsigned short NumChannels;  /* 2 for Stereo */
    unsigned long  SampleRate;  /* 22050 */
    unsigned long  ByteRate;  /* 88200 */
    unsigned short BlockAlign;  /* 4 */
    unsigned short BitsPerSample; /* 16 */
    unsigned long  SubChunk2ID;  /* data */
    unsigned long  SubChunk2Size; /* variable, will set this later */
}
WAV_HEADER;       /* 44 bytes */

WAV_HEADER FAKE_HEADER = { 0x46464952, 0x00000000, 0x45564157, 0x20746D66, 0x00000010,
                           0x0001, 0x0002, 0x00005622, 0x00015888, 0x0004, 0x0010,
                           0x61746164, 0x00000000 };

void wav_output(FILE *musicdat)
{
    const unsigned long POSITION[] =
        {
            0,
            9838576,
            23828720,
            33274480,
            43545200,
            53122640,
            63134608,
            76862256,
            92508192,
            101179808,
            111809812
        };

    FILE *output, *playlist;
    unsigned char output_name[120];
    unsigned char buffer[4096];
    unsigned char name_temp[120];
    unsigned long bytes;
    unsigned char loop;
    unsigned long size;

    /* try to open playlist file, notify errors and exit */
    if (NULL == (playlist = fopen("Kurt Harland - Soul Reaver.m3u", "wb")))
    {
        printf("ERROR: Error creating playlist file.\n");
        exit(-1);
    }

    for (loop = 0; loop < 10; loop++)
    {
        /* create name & write it to the playlist */
        strcpy(output_name, "Track ");
        itoa(loop+1, name_temp, 10);
        strcat(output_name, name_temp);
        strcpy(name_temp, output_name);
        strcat(output_name, ".wav");

        fputs(output_name, playlist);
        fputs("\n", playlist);

        /* try to open output file, notify errors and exit */
        if (NULL == (output = fopen(output_name, "wb")))
        {
            printf("ERROR: Error opening output file %s\n",output_name);
            exit(-1);
        }

        size = POSITION[loop+1] - POSITION[loop];
        fseek(musicdat, POSITION[loop], SEEK_SET);

        printf("Writting file : \"%s\" ... ", output_name);

        /* build and write the WAV header */
        FAKE_HEADER.ChunkSize = size + 36;
        FAKE_HEADER.SubChunk2Size = size;

        if (1 != fwrite(&FAKE_HEADER, sizeof(FAKE_HEADER), 1, output))
        {
            printf("\nERROR: I/O Error writting output file %s\n",output_name);
            exit(-1);
        }

        /* read PCM samples in 4096 blocks if we can & write it */
        while (size > 0)
        {
            if (size > 4096)
            {
                /* read PCM samples */
                if (4096 != (bytes = fread(buffer,1,4096,musicdat)))
                {
                    printf("\nERROR: I/O Error reading music.dat\n");
                    exit(-1);
                }
                size -= 4096;
            }
            else
            {
                if (size != (bytes = fread(buffer,1,size,musicdat)))
                {
                    printf("\nERROR: I/O Error reading music.dat\n");
                    exit(-1);
                }
                size = 0;
            }

            printf("%3d %%\b\b\b\b\b", (int) ((((POSITION[loop+1] - POSITION[loop]) - size) * 100) / (POSITION[loop+1] - POSITION[loop])));
            
            /* write samples */
            if (bytes != fwrite(buffer, 1, bytes, output))
            {
                printf("\nERROR: I/O Error writting output file %s\n",output_name);
                exit(-1);
            }
        }

        /* close the output file */
        if (fclose(output) == EOF)
        {
            printf("\nERROR: Error closing output file %s\n", output_name);
        }

        printf("OK!  \n");

        /* free memory */
        free(buffer);
    }

    printf("Writting M3U playlist ... OK!\n");

    /* close the playlist ile */
    if (fclose(playlist) == EOF)
    {
        printf("ERROR: Error closing playlist file.");
    }

    return;
}
