/*------------------------------------------------------------------------
 Module:        sha-1.h
 Author:        (c) 2003 Felix de las Pozas Alvarez
 Project:       SHA-1
 State:         v1.3 Final version
 Creation Date: December 2 2003
 Description:   Calculates the "Secure Hash Algorithm SHA-1"
                of (a) given file(s). Based on the algorithm
                described in "The Handbook Of Applied
                Cryptography" and the Federal Information
                Processing Standarts publication of august 1
                2002. Header file.
 License:       GNU Public License 
 
    This file is part of SHA-1.
    SHA-1 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    SHA-1 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with SHA-1; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
    USA
------------------------------------------------------------------------*/

#ifndef _SHA_1_H_
#define _SHA_1_H_

#include <stdio.h>

typedef enum {_error_ = -1, _success_, _false_ = 0, _true_} booleantype;

void sha_1_init(void);

booleantype sha_1_file_small_buffer(FILE *filename,
                                    unsigned long long message_length);

booleantype sha_1_file_big_buffer(FILE *filename,
                                  unsigned long long message_length);

void sha_1_buffer(unsigned char *buffer,
                  unsigned long long message_length);

/* SHA-1 160 bit intermediate and final value */
unsigned long chaining_vars[5];

#endif
