/*------------------------------------------------------------------------
 Module:        sr_music.c
 Author:        (c) 2003 Felix de las Pozas Alvarez
 Project:       Soul Reaver 1 music extractor and converter
 State:         v1.1 FINAL VERSION
 Creation Date: December 2 2003
 Description:   Program to extract and convert the music of the game
 				Soul Reaver to WAV, MP3 and OGG formats. Source file.
 License: 		GNU Public License 
	
	This file is part of SR_Music.
    SR_Music free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    SR_Music is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with SHA-1; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
	USA
------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include <io.h>
#include <stdlib.h>
#include "sha-1.h"
#include "output.h"

#define VERSION "v1.1"
#define DATE "December 2 2003"

/* SHA-1 160-bit for music.dat */
const unsigned long SHA1_MUSIC_DAT[] =
    {
        0x52FCE24A, 0x2DF7AE84, 0x699F3D67, 0x13B4A1D8, 0xB43A31F7
    };

int main(const int argc, const char *argv[])
{
    unsigned long long filesize;
	unsigned char loop;
    FILE *musicdat;

	/* The egocentric banner */
	printf("Soul Reaver Music Extractor %s\n", VERSION);
	printf("(c)Felix de las Pozas Alvarez, %s\n\n", DATE);
    
	/* Test the number of arguments, notify errors and exit*/
	if (argc != 3)
	{
		printf("Usage : SR_MUSIC ouput_format musicdat_location\n\n");
		printf("Possible output formats : WAV OGG and MP3\n");
		printf("Read the Usage.txt file for detailed instructions and examples.\n");
		exit(-1);
	}

    /* check if the format argument is valid */
    if (!(!stricmp("wav",argv[1]) || !stricmp("ogg", argv[1]) || !stricmp("mp3", argv[1])))
    {
        printf("ERROR: \"%s\" is an invalid format.\n", argv[1]);
        exit(1);
    }
    
	/* try to open, notify errors and exit */
	if (NULL == (musicdat = fopen(argv[2], "rb")))
	{
		printf("ERROR: Error opening %s\n",argv[2]);
		exit(-1);
	}

	/* test if its the correct file */
	printf("Testing the integrity of the music file ... ");

    fseek(musicdat, 0, SEEK_END);
    filesize = ftell(musicdat);
    fseek(musicdat, 0, SEEK_SET);
    
	sha_1_init();
	if (_success_ != sha_1_file_big_buffer(musicdat, filesize))
    {
        printf("Failed!\nERROR: Error reading %s\n", argv[2]);
        exit(-1);
    }

	for (loop = 0; loop < 5; loop++)
		if (chaining_vars[loop] != SHA1_MUSIC_DAT[loop])
		{
			printf("Failed!\n");
			printf("ERROR: Music file is corrupted or incorrect, please reinstall Soul Reaver.\n");
			exit(-1);
		}

	printf("OK!  \n");
	
	/* write the output files in the selected format */
    
    if (!stricmp("WAV", argv[1]))
        wav_output(musicdat);
    
    if (!stricmp("OGG", argv[1]))
        ogg_output(musicdat);
    
    if (!stricmp("MP3", argv[1]))
        mp3_output(musicdat);
	
	/* Close the file*/
	if (fclose(musicdat) == EOF)
	{
		printf("ERROR: Error closing the file\n");
		exit(-1);
	}
    
    printf("\nFinished! Enjoy the music!\n");

	return 0;
}

